@@ -3,13 +3,17 @@ import logging
 import json
 import functools
 import re
+import traceback
+
+import requests
 
 from datetime import datetime
 from urllib.parse import urljoin, urlsplit, urlunsplit, SplitResult
 
-import requests
+from opentracing_utils import trace, extract_span_from_kwargs
 
 from zmon_cli import __version__
+from zmon_cli.config import DEFAULT_TIMEOUT
 
 
 API_VERSION = 'v1'
@@ -24,7 +28,7 @@ CHECK_DEF = 'check-definitions'
 DASHBOARD = 'dashboard'
 DOWNTIME = 'downtimes'
 ENTITIES = 'entities'
-GRAFANA = 'grafana2-dashboards'
+GRAFANA = 'visualization/dashboards'
 GROUPS = 'groups'
 MEMBER = 'member'
 PHONE = 'phone'
@@ -35,13 +39,13 @@ TOKENS = 'onetime-tokens'
 ALERT_DETAILS_VIEW_URL = '#/alert-details/'
 CHECK_DEF_VIEW_URL = '#/check-definitions/view/'
 DASHBOARD_VIEW_URL = '#/dashboards/views/'
-GRAFANA_DASHBOARD_URL = 'grafana/dashboard/db/'
+GRAFANA_DASHBOARD_URL = 'visualization/dashboard/'
 TOKEN_LOGIN_URL = 'tv/'
 
 logger = logging.getLogger(__name__)
 
 parentheses_re = re.compile('[(]+|[)]+')
-invalid_entity_id_re = re.compile('[^a-zA-Z0-9-@_.\[\]\:]+')
+invalid_entity_id_re = re.compile('[^a-zA-Z0-9-@_.\\[\\]\\:]+')
 
 
 class JSONDateEncoder(json.JSONEncoder):
@@ -66,7 +70,7 @@ def logged(f):
     def wrapper(*args, **kwargs):
         try:
             return f(*args, **kwargs)
-        except:
+        except Exception:
             logger.error('ZMON client failed in: {}'.format(f.__name__))
             raise
 
@@ -83,7 +87,7 @@ def compare_entities(e1, e2):
 
         return (json.loads(json.dumps(e1_copy, cls=JSONDateEncoder)) ==
                 json.loads(json.dumps(e2_copy, cls=JSONDateEncoder)))
-    except:
+    except Exception:
         # We failed during json serialiazation/deserialization, fallback to *not-equal*!
         logger.exception('Failed in `compare_entities`')
         return False
@@ -119,7 +123,8 @@ class Zmon:
     """
 
     def __init__(
-            self, url, token=None, username=None, password=None, timeout=10, verify=True, user_agent=ZMON_USER_AGENT):
+            self, url, token=None, username=None, password=None, timeout=DEFAULT_TIMEOUT, verify=True,
+            user_agent=ZMON_USER_AGENT):
         """Initialize ZMON client."""
         self.timeout = timeout
 
@@ -129,7 +134,7 @@ class Zmon:
 
         self._session = requests.Session()
 
-        self._session.timeout = timeout
+        self._timeout = timeout
         self.user_agent = user_agent
 
         if username and password and token is None:
@@ -249,7 +254,9 @@ class Zmon:
         :return: Deeplink to Grafana dashboard.
         :rtype: str
         """
-        return self.endpoint(GRAFANA_DASHBOARD_URL, dashboard['id'], base_url=self.base_url)
+        if dashboard.get('id', None):
+            return self.endpoint(GRAFANA_DASHBOARD_URL, dashboard['id'], base_url=self.base_url, trailing_slash=False)
+        return ""
 
     @logged
     def status(self) -> dict:
@@ -259,7 +266,7 @@ class Zmon:
         :return: ZMON status.
         :rtype: dict
         """
-        resp = self.session.get(self.endpoint(STATUS))
+        resp = self.session.get(self.endpoint(STATUS), timeout=self._timeout)
 
         return self.json(resp)
 
@@ -267,8 +274,9 @@ class Zmon:
 # ENTITIES
 ########################################################################################################################
 
+    @trace(pass_span=True)
     @logged
-    def get_entities(self, query=None) -> list:
+    def get_entities(self, query=None, **kwargs) -> list:
         """
         Get ZMON entities, with optional filtering.
 
@@ -282,14 +290,18 @@ class Zmon:
         query_str = json.dumps(query) if query else ''
         logger.debug('Retrieving entities with query: {} ...'.format(query_str))
 
+        current_span = extract_span_from_kwargs(**kwargs)
+        current_span.log_kv({'query': query_str})
+
         params = {'query': query_str} if query else None
 
-        resp = self.session.get(self.endpoint(ENTITIES), params=params)
+        resp = self.session.get(self.endpoint(ENTITIES), params=params, timeout=self._timeout)
 
         return self.json(resp)
 
+    @trace(pass_span=True)
     @logged
-    def get_entity(self, entity_id: str) -> str:
+    def get_entity(self, entity_id: str, **kwargs) -> str:
         """
         Retrieve single entity.
 
@@ -301,11 +313,15 @@ class Zmon:
         """
         logger.debug('Retrieving entities with id: {} ...'.format(entity_id))
 
-        resp = self.session.get(self.endpoint(ENTITIES, entity_id, trailing_slash=False))
+        current_span = extract_span_from_kwargs(**kwargs)
+        current_span.set_tag('entity_id', entity_id)
+
+        resp = self.session.get(self.endpoint(ENTITIES, entity_id, trailing_slash=False), timeout=self._timeout)
         return self.json(resp)
 
+    @trace(pass_span=True)
     @logged
-    def add_entity(self, entity: dict) -> requests.Response:
+    def add_entity(self, entity: dict, **kwargs) -> requests.Response:
         """
         Create or update an entity on ZMON.
 
@@ -327,14 +343,19 @@ class Zmon:
 
         logger.debug('Adding new entity: {} ...'.format(entity['id']))
 
-        resp = self.session.put(self.endpoint(ENTITIES, trailing_slash=False), json=entity)
+        current_span = extract_span_from_kwargs(**kwargs)
+        current_span.set_tag('entity_id', entity['id'])
+
+        data = json.dumps(entity, cls=JSONDateEncoder)
+        resp = self.session.put(self.endpoint(ENTITIES, trailing_slash=False), data=data, timeout=self._timeout)
 
         resp.raise_for_status()
 
         return resp
 
+    @trace(pass_span=True)
     @logged
-    def delete_entity(self, entity_id: str) -> bool:
+    def delete_entity(self, entity_id: str, **kwargs) -> bool:
         """
         Delete entity from ZMON.
 
@@ -350,6 +371,9 @@ class Zmon:
         """
         logger.debug('Removing existing entity: {} ...'.format(entity_id))
 
+        current_span = extract_span_from_kwargs(**kwargs)
+        current_span.set_tag('entity_id', entity_id)
+
         resp = self.session.delete(self.endpoint(ENTITIES, entity_id))
 
         resp.raise_for_status()
@@ -360,8 +384,9 @@ class Zmon:
 # DASHBOARD
 ########################################################################################################################
 
+    @trace(pass_span=True)
     @logged
-    def get_dashboard(self, dashboard_id: str) -> dict:
+    def get_dashboard(self, dashboard_id: str, **kwargs) -> dict:
         """
         Retrieve a ZMON dashboard.
 
@@ -371,12 +396,16 @@ class Zmon:
         :return: Dashboard dict.
         :rtype: dict
         """
-        resp = self.session.get(self.endpoint(DASHBOARD, dashboard_id))
+        current_span = extract_span_from_kwargs(**kwargs)
+        current_span.set_tag('dashboard_id', dashboard_id)
+
+        resp = self.session.get(self.endpoint(DASHBOARD, dashboard_id), timeout=self._timeout)
 
         return self.json(resp)
 
+    @trace(pass_span=True)
     @logged
-    def update_dashboard(self, dashboard: dict) -> dict:
+    def update_dashboard(self, dashboard: dict, **kwargs) -> dict:
         """
         Create or update dashboard.
 
@@ -388,14 +417,16 @@ class Zmon:
         :return: Dashboard dict.
         :rtype: dict
         """
+        current_span = extract_span_from_kwargs(**kwargs)
         if 'id' in dashboard and dashboard['id']:
             logger.debug('Updating dashboard with ID: {} ...'.format(dashboard['id']))
+            current_span.set_tag('dashboard_id', dashboard['id'])
 
-            resp = self.session.post(self.endpoint(DASHBOARD, dashboard['id']), json=dashboard)
+            resp = self.session.post(self.endpoint(DASHBOARD, dashboard['id']), json=dashboard, timeout=self._timeout)
         else:
             # new dashboard
             logger.debug('Adding new dashboard ...')
-            resp = self.session.post(self.endpoint(DASHBOARD), json=dashboard)
+            resp = self.session.post(self.endpoint(DASHBOARD), json=dashboard, timeout=self._timeout)
 
         resp.raise_for_status()
 
@@ -405,8 +436,9 @@ class Zmon:
 # CHECK-DEFS
 ########################################################################################################################
 
+    @trace(pass_span=True)
     @logged
-    def get_check_definition(self, definition_id: int) -> dict:
+    def get_check_definition(self, definition_id: int, **kwargs) -> dict:
         """
         Retrieve check defintion.
 
@@ -416,7 +448,10 @@ class Zmon:
         :return: Check definition dict.
         :rtype: dict
         """
-        resp = self.session.get(self.endpoint(CHECK_DEF, definition_id))
+        current_span = extract_span_from_kwargs(**kwargs)
+        current_span.set_tag('check_id', definition_id)
+
+        resp = self.session.get(self.endpoint(CHECK_DEF, definition_id), timeout=self._timeout)
 
         # TODO: total hack! API returns 200 if check def does not exist!
         if resp.text == '':
@@ -425,6 +460,7 @@ class Zmon:
 
         return self.json(resp)
 
+    @trace()
     @logged
     def get_check_definitions(self) -> list:
         """
@@ -433,12 +469,13 @@ class Zmon:
         :return: List of check-defs.
         :rtype: list
         """
-        resp = self.session.get(self.endpoint(ACTIVE_CHECK_DEF))
+        resp = self.session.get(self.endpoint(ACTIVE_CHECK_DEF), timeout=self._timeout)
 
         return self.json(resp).get('check_definitions')
 
+    @trace(pass_span=True)
     @logged
-    def update_check_definition(self, check_definition: dict) -> dict:
+    def update_check_definition(self, check_definition, skip_validation=False, **kwargs) -> dict:
         """
         Update existing check definition.
 
@@ -447,23 +484,36 @@ class Zmon:
         :param check_definition: ZMON check definition dict.
         :type check_definition: dict
 
+        :param skip_validation: Skip validation of the check command syntax.
+        :type skip_validation: bool
+
         :return: Check definition dict.
         :rtype: dict
         """
+        current_span = extract_span_from_kwargs(**kwargs)
         if 'owning_team' not in check_definition:
+            current_span.set_tag('error', True)
+            current_span.log_kv({'exception': 'Check definition must have "owning_team"'})
             raise ZmonArgumentError('Check definition must have "owning_team"')
 
         if 'status' not in check_definition:
             check_definition['status'] = 'ACTIVE'
 
-        self.validate_check_command(check_definition['command'])
+        if not skip_validation:
+            try:
+                self.validate_check_command(check_definition['command'])
+            except Exception:
+                current_span.set_tag('error', True)
+                current_span.log_kv({'exception': traceback.format_exc()})
+                raise
 
-        resp = self.session.post(self.endpoint(CHECK_DEF), json=check_definition)
+        resp = self.session.post(self.endpoint(CHECK_DEF), json=check_definition, timeout=self._timeout)
 
         return self.json(resp)
 
+    @trace(pass_span=True)
     @logged
-    def delete_check_definition(self, check_definition_id: int) -> requests.Response:
+    def delete_check_definition(self, check_definition_id: int, **kwargs) -> requests.Response:
         """
         Delete existing check definition.
 
@@ -473,6 +523,9 @@ class Zmon:
         :return: HTTP response.
         :rtype: :class:`requests.Response`
         """
+        current_span = extract_span_from_kwargs(**kwargs)
+        current_span.set_tag('check_id', str(check_definition_id))
+
         resp = self.session.delete(self.endpoint(CHECK_DEF, check_definition_id))
 
         resp.raise_for_status()
@@ -483,8 +536,9 @@ class Zmon:
 # ALERT-DEFS & DATA
 ########################################################################################################################
 
+    @trace(pass_span=True)
     @logged
-    def get_alert_definition(self, alert_id: int) -> dict:
+    def get_alert_definition(self, alert_id: int, **kwargs) -> dict:
         """
         Retrieve alert definition.
 
@@ -494,10 +548,14 @@ class Zmon:
         :return: Alert definition dict.
         :rtype: dict
         """
-        resp = self.session.get(self.endpoint(ALERT_DEF, alert_id))
+        current_span = extract_span_from_kwargs(**kwargs)
+        current_span.set_tag('alert_id', str(alert_id))
+
+        resp = self.session.get(self.endpoint(ALERT_DEF, alert_id), timeout=self._timeout)
 
         return self.json(resp)
 
+    @trace()
     @logged
     def get_alert_definitions(self) -> list:
         """
@@ -506,12 +564,13 @@ class Zmon:
         :return: List of alert-defs.
         :rtype: list
         """
-        resp = self.session.get(self.endpoint(ACTIVE_ALERT_DEF))
+        resp = self.session.get(self.endpoint(ACTIVE_ALERT_DEF), timeout=self._timeout)
 
         return self.json(resp).get('alert_definitions')
 
+    @trace(pass_span=True)
     @logged
-    def create_alert_definition(self, alert_definition: dict) -> dict:
+    def create_alert_definition(self, alert_definition: dict, **kwargs) -> dict:
         """
         Create new alert definition.
 
@@ -524,21 +583,28 @@ class Zmon:
         :return: Alert definition dict.
         :rtype: dict
         """
+        current_span = extract_span_from_kwargs(**kwargs)
         if 'last_modified_by' not in alert_definition:
+            current_span.set_tag('error', True)
+            current_span.log_kv({'exception': 'Alert definition must have "last_modified_by"'})
             raise ZmonArgumentError('Alert definition must have "last_modified_by"')
 
         if 'status' not in alert_definition:
             alert_definition['status'] = 'ACTIVE'
 
         if 'check_definition_id' not in alert_definition:
+            current_span.set_tag('error', True)
+            current_span.log_kv({'exception': 'Alert definition must have "last_modified_by"'})
             raise ZmonArgumentError('Alert defintion must have "check_definition_id"')
+        current_span.set_tag('check_id', alert_definition['check_definition_id'])
 
-        resp = self.session.post(self.endpoint(ALERT_DEF), json=alert_definition)
+        resp = self.session.post(self.endpoint(ALERT_DEF), json=alert_definition, timeout=self._timeout)
 
         return self.json(resp)
 
+    @trace(pass_span=True)
     @logged
-    def update_alert_definition(self, alert_definition: dict) -> dict:
+    def update_alert_definition(self, alert_definition: dict, **kwargs) -> dict:
         """
         Update existing alert definition.
 
@@ -551,25 +617,35 @@ class Zmon:
         :return: Alert definition dict.
         :rtype: dict
         """
+        current_span = extract_span_from_kwargs(**kwargs)
         if 'last_modified_by' not in alert_definition:
+            current_span.set_tag('error', True)
+            current_span.log_kv({'exception': 'Alert definition must have "last_modified_by"'})
             raise ZmonArgumentError('Alert definition must have "last_modified_by"')
 
         if 'id' not in alert_definition:
+            current_span.set_tag('error', True)
+            current_span.log_kv({'exception': 'Alert definition must have "id"'})
             raise ZmonArgumentError('Alert definition must have "id"')
+        current_span.set_tag('alert_id', alert_definition['id'])
 
         if 'check_definition_id' not in alert_definition:
+            current_span.set_tag('error', True)
+            current_span.log_kv({'exception': 'Alert definition must have "check_definition_id"'})
             raise ZmonArgumentError('Alert defintion must have "check_definition_id"')
+        current_span.set_tag('check_id', alert_definition['check_definition_id'])
 
         if 'status' not in alert_definition:
             alert_definition['status'] = 'ACTIVE'
 
         resp = self.session.put(
-            self.endpoint(ALERT_DEF, alert_definition['id']), json=alert_definition)
+            self.endpoint(ALERT_DEF, alert_definition['id']), json=alert_definition, timeout=self._timeout)
 
         return self.json(resp)
 
+    @trace(pass_span=True)
     @logged
-    def delete_alert_definition(self, alert_definition_id: int) -> dict:
+    def delete_alert_definition(self, alert_definition_id: int, **kwargs) -> dict:
         """
         Delete existing alert definition.
 
@@ -579,12 +655,15 @@ class Zmon:
         :return: Alert definition dict.
         :rtype: dict
         """
+        current_span = extract_span_from_kwargs(**kwargs)
+        current_span.set_tag('alert_id', str(alert_definition_id))
         resp = self.session.delete(self.endpoint(ALERT_DEF, alert_definition_id))
 
         return self.json(resp)
 
+    @trace(pass_span=True)
     @logged
-    def get_alert_data(self, alert_id: int) -> dict:
+    def get_alert_data(self, alert_id: int, **kwargs) -> dict:
         """
         Retrieve alert data.
 
@@ -606,7 +685,9 @@ class Zmon:
                 "entity-id-3": 100
             }
         """
-        resp = self.session.get(self.endpoint(ALERT_DATA, alert_id, 'all-entities'))
+        current_span = extract_span_from_kwargs(**kwargs)
+        current_span.set_tag('alert_id', str(alert_id))
+        resp = self.session.get(self.endpoint(ALERT_DATA, alert_id, 'all-entities'), timeout=self._timeout)
 
         return self.json(resp)
 
@@ -614,8 +695,9 @@ class Zmon:
 # SEARCH
 ########################################################################################################################
 
+    @trace(pass_span=True)
     @logged
-    def search(self, q, teams: list=None) -> dict:
+    def search(self, q, limit=None, teams=None, **kwargs) -> dict:
         """
         Search ZMON dashboards, checks, alerts and grafana dashboards with optional team filtering.
 
@@ -639,17 +721,20 @@ class Zmon:
                 "grafana_dashboards": [{"id": "123", "title": "ZMON grafana", "team": ""}],
             }
         """
-        if not q:
-            raise ZmonArgumentError('No search query value!')
-
+        current_span = extract_span_from_kwargs(**kwargs)
         if teams and type(teams) not in (list, tuple):
+            current_span.set_tag('error', True)
+            current_span.log_kv({'exception': '"teams" should be a list'})
             raise ZmonArgumentError('"teams" should be a list!')
 
         params = {'query': q}
+        if limit:
+            params.update({'limit': limit})
         if teams:
             params['teams'] = ','.join(teams)
 
-        resp = self.session.get(self.endpoint(SEARCH), params=params)
+        current_span.log_kv({'query', json.dumps(params)})
+        resp = self.session.get(self.endpoint(SEARCH), params=params, timeout=self._timeout)
 
         return self.json(resp)
 
@@ -658,6 +743,7 @@ class Zmon:
 # ONETIME-TOKENS
 ########################################################################################################################
 
+    @trace()
     @logged
     def list_onetime_tokens(self) -> list:
         """
@@ -676,10 +762,11 @@ class Zmon:
               created: 2016-08-26 12:51:13.506000
               token: 9pSzKpcO
         """
-        resp = self.session.get(self.endpoint(TOKENS))
+        resp = self.session.get(self.endpoint(TOKENS), timeout=self._timeout)
 
         return self.json(resp)
 
+    @trace()
     @logged
     def get_onetime_token(self) -> str:
         """
@@ -690,7 +777,7 @@ class Zmon:
         :return: One-time token.
         :retype: str
         """
-        resp = self.session.post(self.endpoint(TOKENS), json={})
+        resp = self.session.post(self.endpoint(TOKENS), json={}, timeout=self._timeout)
 
         resp.raise_for_status()
 
@@ -700,27 +787,32 @@ class Zmon:
 # GRAFANA
 ########################################################################################################################
 
+    @trace(pass_span=True)
     @logged
-    def get_grafana_dashboard(self, grafana_dashboard_id: str) -> dict:
+    def get_grafana_dashboard(self, grafana_dashboard_uid: str, **kwargs) -> dict:
         """
         Retrieve Grafana dashboard.
 
-        :param grafana_dashboard_id: Grafana dashboard ID.
-        :type grafana_dashboard_id: str
+        :param grafana_dashboard_uid: Grafana dashboard UID.
+        :type grafana_dashboard_uid: str
 
         :return: Grafana dashboard dict.
         :rtype: dict
         """
-        resp = self.session.get(self.endpoint(GRAFANA, grafana_dashboard_id))
+        current_span = extract_span_from_kwargs(**kwargs)
+        current_span.set_tag('grafana_dashboard_uid', grafana_dashboard_uid)
+        url = self.endpoint(GRAFANA, grafana_dashboard_uid, trailing_slash=False)
+        resp = self.session.get(url, timeout=self._timeout)
 
         return self.json(resp)
 
+    @trace(pass_span=True)
     @logged
-    def update_grafana_dashboard(self, grafana_dashboard: dict) -> dict:
+    def update_grafana_dashboard(self, grafana_dashboard: dict, **kwargs) -> dict:
         """
         Update existing Grafana dashboard.
 
-        Atrributes ``id`` and ``title`` are required.
+        Atrributes ``uid`` and ``title`` are required.
 
         :param grafana_dashboard: Grafana dashboard dict.
         :type grafana_dashboard: dict
@@ -728,12 +820,24 @@ class Zmon:
         :return: Grafana dashboard dict.
         :rtype: dict
         """
-        if 'id' not in grafana_dashboard['dashboard']:
-            raise ZmonArgumentError('Grafana dashboard must have "id"')
+        current_span = extract_span_from_kwargs(**kwargs)
+
+        if 'uid' not in grafana_dashboard['dashboard']:
+            current_span.set_tag('error', True)
+            current_span.log_kv({'exception': 'Grafana dashboard must have "uid". Use Grafana6 dashboard format.'})
+            raise ZmonArgumentError('Grafana dashboard must have "uid". Hint: Use Grafana6 dashboard format.')
+
         elif 'title' not in grafana_dashboard['dashboard']:
+            current_span.set_tag('error', True)
+            current_span.log_kv({'exception': 'Grafana dashboard must have "title"'})
             raise ZmonArgumentError('Grafana dashboard must have "title"')
 
-        resp = self.session.post(self.endpoint(GRAFANA), json=grafana_dashboard)
+        current_span.set_tag('grafana_dashboard_uid', grafana_dashboard['dashboard']['uid'])
+
+        if 'id' in grafana_dashboard['dashboard'] and grafana_dashboard['dashboard']['id'] is not None:
+            current_span.set_tag('grafana_dashboard_id', grafana_dashboard['dashboard']['id'])
+
+        resp = self.session.post(self.endpoint(GRAFANA), json=json.dumps(grafana_dashboard), timeout=self._timeout)
 
         return self.json(resp)
 
@@ -741,8 +845,9 @@ class Zmon:
 # DOWNTIMES
 ########################################################################################################################
 
+    @trace(pass_span=True)
     @logged
-    def create_downtime(self, downtime: dict) -> dict:
+    def create_downtime(self, downtime: dict, **kwargs) -> dict:
         """
         Create a downtime for specific entities.
 
@@ -765,13 +870,23 @@ class Zmon:
                 "end_time": 1473341037.312921,
             }
         """
+        current_span = extract_span_from_kwargs(**kwargs)
         if not downtime.get('entities'):
+            current_span.set_tag('error', True)
+            current_span.log_kv({'exception': 'At least one entity ID should be specified'})
             raise ZmonArgumentError('At least one entity ID should be specified')
 
         if not downtime.get('start_time') or not downtime.get('end_time'):
+            current_span.set_tag('error', True)
+            current_span.log_kv({'exception': 'Downtime must specify "start_time" and "end_time"'})
             raise ZmonArgumentError('Downtime must specify "start_time" and "end_time"')
 
-        resp = self.session.post(self.endpoint(DOWNTIME), json=downtime)
+        current_span.set_tag('entity_ids', str(downtime.get('entities')))
+        # FIXME - those also?
+        # current_span.set_tag('start_time', str(downtime.get('start_time')))
+        # current_span.set_tag('end_time', str(downtime.get('end_time')))
+
+        resp = self.session.post(self.endpoint(DOWNTIME), json=downtime, timeout=self._timeout)
 
         return self.json(resp)
 
@@ -781,7 +896,7 @@ class Zmon:
 
     @logged
     def get_groups(self):
-        resp = self.session.get(self.endpoint(GROUPS))
+        resp = self.session.get(self.endpoint(GROUPS), timeout=self._timeout)
 
         return self.json(resp)
 
@@ -795,7 +910,7 @@ class Zmon:
 
         logger.debug('Switching active user: {}'.format(user_name))
 
-        resp = self.session.put(self.endpoint(GROUPS, group_name, 'active', user_name))
+        resp = self.session.put(self.endpoint(GROUPS, group_name, 'active', user_name), timeout=self._timeout)
 
         if not resp.ok:
             logger.error('Failed to switch active user {}'.format(user_name))
@@ -805,9 +920,9 @@ class Zmon:
 
     @logged
     def add_member(self, group_name, user_name):
-        resp = self.session.put(self.endpoint(GROUPS, group_name, MEMBER, user_name))
+        resp = self.session.put(self.endpoint(GROUPS, group_name, MEMBER, user_name), timeout=self._timeout)
 
-        resp.raise_for_status
+        resp.raise_for_status()
 
         return resp.text == '1'
 
@@ -821,7 +936,7 @@ class Zmon:
 
     @logged
     def add_phone(self, member_email, phone_nr):
-        resp = self.session.put(self.endpoint(GROUPS, member_email, PHONE, phone_nr))
+        resp = self.session.put(self.endpoint(GROUPS, member_email, PHONE, phone_nr), timeout=self._timeout)
 
         resp.raise_for_status()
 
@@ -837,7 +952,7 @@ class Zmon:
 
     @logged
     def set_name(self, member_email, member_name):
-        resp = self.session.put(self.endpoint(GROUPS, member_email, PHONE, member_name))
+        resp = self.session.put(self.endpoint(GROUPS, member_email, PHONE, member_name), timeout=self._timeout)
 
         resp.raise_for_status()
 
