@@ -7,13 +7,15 @@ from zmon_cli.client import ZmonArgumentError
 
 
 @cli.command()
-@click.argument('search_query')
+@click.argument('search_query', default="")
 @click.option('--team', '-t', multiple=True, required=False,
               help='Filter search by team. Multiple teams filtering is supported.')
+@click.option('--limit', '-l', multiple=False, required=False,
+              help='Limit number of results, default is 25')
 @click.pass_obj
 @output_option
 @pretty_json
-def search(obj, search_query, team, output, pretty):
+def search(obj, search_query, team, limit, output, pretty):
     """
     Search dashboards, alerts, checks and grafana dashboards.
 
@@ -25,7 +27,7 @@ def search(obj, search_query, team, output, pretty):
 
     with Output('Searching ...', nl=True, output=output, pretty_json=pretty, printer=render_search) as act:
         try:
-            data = client.search(search_query, teams=team)
+            data = client.search(search_query, limit=limit, teams=team)
 
             for check in data['checks']:
                 check['link'] = client.check_definition_url(check)
